classdef MESH_IO
    %MESH_IO Summary of this class goes here
    %   Detailed explanation goes here
    % TODO: add woff, wvtk_maya
    % TODO: check wavi, wmpeg4
    
    properties
    end
    
    methods (Static)
        
        %%%%%%%%
        % Read %
        %%%%%%%%

        % Only triangles or quads
        function [X,T,UV,Tuv,OBJ] = robj(filename, d)
            % X coords, in n x 3
            % T faces, triangles or quads, in m x 3 or m x 4
            % UV texture coords, in n x 2
            % Tuv texure faces, in mt x 3 or mt x 4
            if nargin < 2
                d = 3;
            end
            UV = []; Tuv = [];
            
            if d == 3
                % TODO: use loadawobj for triangles as well, much faster
                OBJ = read_wobj( filename );
                X = OBJ.vertices;
                T = OBJ.objects(1).data.vertices;
            else
                % ASSUME QUADS!
                OBJ = loadawobj( filename );
                X = OBJ.v';
                T = OBJ.f4';
                if isfield(OBJ,'tc4')
                    Tuv = OBJ.tc4';
                    UV = OBJ.vt';
                end
            end
        end

        function [X,T] = roff(filename, d)
            if nargin < 2
                d = 3;
            end
            
            fid = fopen(filename,'r');
            if( fid==-1 )
                error('Cannot open the file.');
                return;
            end
            
            str = fgets(fid);   % -1 if eof
            
            if strcmp(str(1:4), 'COFF')
                [X,T,~] = readCoff(filename,4); % assume 4 color channels
                return;
            end
            
            if ~strcmp(str(1:3), 'OFF')
                error('The file is not a valid OFF one.');
            end
           
            str = fgets(fid);
            sizes = sscanf(str, '%d %d', 2);
            while length(sizes) ~= 2
                str = fgets(fid);
                sizes = sscanf(str, '%d %d', 2);
            end
            nv = sizes(1);
            nf = sizes(2);
            
            % Read vertices
            [X,cnt] = fscanf(fid,'%lf %lf %lf\n', [3,nv]);
            if cnt~=3*nv
                warning('Problem in reading vertices.');
            end
            X = X';
            
            if d == 3
                [T,~] = fscanf(fid,'3 %ld %ld %ld\n', [3,inf]);
            elseif d == 4
                [T,~] = fscanf(fid,'4 %ld %ld %ld %ld\n', [4,inf]);
            else
                error('unsupported faces');
            end
                    
            T = T'+1;
            
            fclose(fid);
        end
        
        function [ V ] = rnrosy(filename)
            fid = fopen(filename,'r');
            if( fid==-1 )
                error('Cannot open the file.');
                return;
            end
            
            str = fgets(fid);
            sizes = sscanf(str, '%d %d', 2);
            nv = sizes(1);
            
            [V,~] = fscanf(fid,'%lf %lf %lf', [3,nv]);
            
            fclose(fid);
        end
        
        function [ XF ] = rffield(filename)
            fid = fopen( [filename '.ffield'], 'r' );
            if( fid == -1 )
                error('Cannot open the file.');
                return;
            end
            
            str = fgets(fid); % Thresholds
            str = fgets(fid); % ??
            str = fgets(fid); % number of faces
            nf = sscanf(str,'%d');
            str = fgets(fid); % k1 k2 ...
            
            % read cross field
            [XF,cnt] = fscanf(fid,'%lf %lf %lf %lf %lf %lf %lf %lf\n', [8,nf]);
            XF = XF'; XF = XF(:,3:end);
            
            fclose( fid );
        end
        %%%%%%%%%
        % Write %
        %%%%%%%%%

        % image files
        function wfigs(filename,mesh,F,varargin)
        
            p = inputParser;
            addOptional(p,'Titles',[]);
            addOptional(p,'TitlesFontSize',[]);
            addOptional(p,'Colormap','jet');
            addOptional(p,'Colorbar',[]);
            addOptional(p,'CAxis','auto');
            addOptional(p,'CAxisV',[]);
            addOptional(p,'Plot','func');
            addOptional(p,'Montage',[1,1]);
            addOptional(p,'View',[]);
            addOptional(p,'Zoom',1);
            addOptional(p,'UseZoom',0);
            addOptional(p,'Resolution',1024);
            addOptional(p,'Camera',[]);
            addOptional(p,'OpenGL',0);
            addOptional(p,'LightPos',[0 0 1]);
            addOptional(p,'UVt',[]);
            addOptional(p,'FUVt',[]);
            addOptional(p,'SingularPts',0);
            addOptional(p,'SingularPtsR',20);
            addOptional(p,'AspectRatio',1);
            addOptional(p,'EdgeColor','k');
            addOptional(p,'TightInset',1);
            parse(p,varargin{:});
            
            if strcmp(p.Results.Plot,'mesh') == 1, sz = 1; 
            elseif strcmp(p.Results.Plot,'func') == 1, sz = size(F,2); end;
            
            for i = 1:sz
                figure;
                if strcmp(p.Results.Plot,'mesh') == 1
                    
                    MESH_VIS.mesh( mesh, ...
                                   'CW',F,...
                                   'UVt',p.Results.UVt,...
                                   'FUVt',p.Results.FUVt,...
                                   'SingularPts',p.Results.SingularPts,...
                                   'SingularPtsR',p.Results.SingularPtsR,...
                                   'EdgeColor',p.Results.EdgeColor);
                    
                elseif strcmp(p.Results.Plot,'func') == 1; 
                    
                    h = MESH_VIS.func( mesh, F(:,i),...
                                   'Colormap', p.Results.Colormap );
                    colorbar off;
                    if strcmp(p.Results.CAxis,'scaled') == 1
                        ca = [min(min(F)),max(max(F))];
                        caxis( ca );
                    elseif strcmp(p.Results.CAxis,'manual') == 1
                        caxis(p.Results.CAxisV);
                    end
                    if size(F,1) == mesh.nv
                        set(gca,'xlim',[min(mesh.vertices(:,1)),max(mesh.vertices(:,1))]);
                        set(gca,'ylim',[min(mesh.vertices(:,2)),max(mesh.vertices(:,2))]);
                        set(gca,'zlim',[min(mesh.vertices(:,3)),max(mesh.vertices(:,3))]);
                    elseif size(F,1) == mesh.nv_uv
                        set(gca,'xlim',[0 1]);
                        set(gca,'ylim',[0 1]);
                    end
                    if (p.Results.TightInset == 1)
                        set(gca, 'LooseInset', get(gca, 'TightInset'));
                    end
                end
                
                if strcmp(p.Results.Colorbar, 'on')
                    colorbar;
                end
                
                if isempty(p.Results.Camera) == 0
                    MESH_VIS.set_camera(gca,p.Results.Camera);
                end
                
                if p.Results.OpenGL == 1
                    lighting phong;
                    material dull;
%                     light('Position',p.Results.LightPos,...
%                           'Style','local','Color',[.8,.8,.8]);
                    camlight('headlight');
                end
                
                if numel(p.Results.Titles) > 0
                    if isempty(p.Results.TitlesFontSize)
                        FontSize = 25;
                    else
                        FontSize = p.Results.TitlesFontSize;
                    end
                    title(p.Results.Titles{i},'FontSize',FontSize,...
                          'interpreter','latex');    
                end
                
                if (p.Results.UseZoom)
                    zoom(p.Results.Zoom);   
                end
                
                pngname = sprintf('%s_%04d.png',filename,i);
                
                dpi = get(0, 'ScreenPixelsPerInch');
                in = p.Results.Resolution/dpi;
                 
                fig = gcf;
                
                fig.WindowStyle = 'normal';
                % set all units inside figure to normalized so that everything is scaling accordingly
                set(findall(fig,'Units','pixels'),'Units','normalized');
                % do not show figure on screen
                set(fig, 'visible', 'off')
                % set figure units to pixels & adjust figure size
                fig.Units = 'pixels';
                fig.OuterPosition = [0 0 p.Results.Resolution p.Results.Resolution];
                
                fig.PaperPositionMode = 'manual';
                fig.PaperUnits = 'inches';
                fig.PaperPosition = [0 0 p.Results.AspectRatio*in in];
                set(gca,'DataAspectRatio',[1 1 1])
                print(pngname,'-dpng',sprintf('-r%d', dpi))
                close all;
            end
            
            % montage
            if p.Results.Montage(1) == 1
                rows = p.Results.Montage(2);
                cols = size(F,2) / rows;

                A = [];
                for i = 1:rows
                    b = [];
                    for j = 1:cols
                        pngname = sprintf('%s_%04d.png',filename,cols*(i-1)+j);
                        a = imread(pngname);
                        b = cat(2,b,a);
                    end
                    A = cat(1,A,b);
                end
                imwrite(A,[filename '.png']);
                delete( sprintf('%s_*.png',filename) );
                close all;
            end
            
%             set(gca,'FontSize',15)
        end
        
        % geometry files
        function wobj(filename,X,T,UV,T_uv,imfname)
            usetexturecoords = 0;
            usematerial = 0;
            objid = 1;
            
            if nargin > 3
                usetexturecoords = 1;
            end
            if nargin > 5
                usematerial = 1;
            end
            if usematerial
                 % Set material
                material(1).type='newmtl';
                material(1).data='default';
                material(2).type='Ka';
                material(2).data=[0.8 0.4 0.4];
                material(3).type='Kd';
                material(3).data=[0.8 0.4 0.4];
                material(4).type='Ks';
                material(4).data=[1 1 1];
                material(5).type='illum';
                material(5).data=2;
                material(6).type='Ns';
                material(6).data=27;
                material(6).type='map_Kd';
                material(6).data=imfname;
                OBJ.material = material;
                OBJ.objects(objid).type='usemtl';
                OBJ.objects(objid).data='default';
                objid = objid + 1;
            end
                        
            OBJ.vertices = X;
            OBJ.objects(objid).data.vertices = T;
            OBJ.objects(objid).type = 'f';
            if usetexturecoords 
                OBJ.vertices_texture = UV;
                OBJ.objects(objid).data.texture = T_uv;            
            end
            
            write_wobj( OBJ, filename );
        end
        
        function woff(filename,X,T,varargin)
            fid = fopen(filename,'w');
            if( fid==-1 )
                error('Cannot open the file.');
                return;
            end

            nv = size(X,1);
            nf = size(T,1);

            fprintf(fid, 'OFF\r\n');
            fprintf(fid, '%d %d 0\r\n', nv, nf);

            fprintf(fid, '%.9f %.9f %.9f\r\n', X');
            fprintf(fid, '3 %d %d %d\r\n', (T-1)');

            fclose(fid);
        end

        function woffq(filename,X,Q,varargin)
            fid = fopen(filename,'w');
            if( fid==-1 )
                error('Cannot open the file.');
                return;
            end

            nv = size(X,1);
            nf = size(Q,1);

            fprintf(fid, 'OFF\r\n');
            fprintf(fid, '%d %d 0\r\n', nv, nf);

            fprintf(fid, '%.9f %.9f %.9f\r\n', X');
            fprintf(fid, '4 %d %d %d %d\r\n', (Q-1)');

            fclose(fid);
        end
        
        function wnrosy(filename,M,V,varargin)
            p = inputParser;
            addOptional(p,'n',1);
            parse(p,varargin{:});
            
%             fid = fopen(filename,'w');
%             if( fid==-1 )
%                 error('Cannot open the file.');
%                 return;
%             end
%             
%             nv = size(V,1);
%             fprintf(fid,'%d %d\n', nv, p.Results.n);
%             fprintf(fid,'%.9f %.9f %.9f\r\n', V');
%             
%             fclose(fid);
            % Based on Danielle's code
            export_to_lic(filename,M,V);
        end
        
        function wvf(filename, V)
            fid = fopen(filename,'w');
            if( fid==-1 )
                error('Cannot open the file.');
                return;
            end
            
            fprintf(fid,'%.9f %.9f %.9f\r\n', V');
            
            fclose(fid);
            
        end

        function wframe(filename, V1, V2)
            fid = fopen(filename,'w');
            if( fid==-1 )
                error('Cannot open the file.');
                return;
            end
            
            fprintf(fid,'%.9f %.9f %.9f %.9f %.9f %.9f\r\n', [V1,V2]');
            
            fclose(fid);
        end

        
        function wvtk(filename,X,T,varargin)
            p = inputParser;

            addOptional(p,'Xn',[]);
            addOptional(p,'func',[]);
            addOptional(p,'func_str','vc');
            addOptional(p,'timestep',0);
            addOptional(p,'CM',[]);

            parse(p,varargin{:});
            
            nv = length(X);
            nf = length(T);

            fid = fopen(filename,'w');

            % standard header
            fprintf(fid,'# vtk DataFile Version 3.0\n');
            fprintf(fid,'vtk output\n');
            fprintf(fid,'ASCII\n');
            fprintf(fid,'DATASET POLYDATA\n');

            % if available, add time annotation
            if isempty( p.Results.timestep ) == 0
                fprintf(fid,'FIELD FieldData 1\n');
                fprintf(fid,'TIME 1 1 double\n');
                fprintf(fid,'%g\n',p.Results.timestep);
            end

            % geometry data
            fprintf(fid,'POINTS %d float\n', nv);
            fprintf(fid,'%g %g %g\n', X');
            fprintf(fid,'POLYGONS %d %d\n', nf, 4*nf);
            fprintf(fid,'3 %d %d %d\n', T'-1);
            fprintf(fid,'\n');

            % add point data
            if isempty( p.Results.func ) == 0 || isempty( p.Results.Xn ) == 0
                fprintf(fid,'POINT_DATA %d\n', nv);

                if isempty( p.Results.func ) == 0
                    fprintf(fid,'SCALARS %s float 1\n', p.Results.func_str);
                    lt = 'default'; 
                    if isempty( p.Results.CM ) == 0; lt = 'lt'; end;
                    fprintf(fid,'LOOKUP_TABLE %s\n', lt);
                    fprintf(fid,'%g\n', p.Results.func);
                end

                if isempty( p.Results.Xn ) == 0
                    fprintf(fid,'NORMALS point_normals float\n');
                    fprintf(fid,'%g %g %g\n', p.Results.Xn');
                end

                fprintf(fid,'\n');
            end

            if isempty( p.Results.CM ) == 0
                fprintf(fid,'LOOKUP_TABLE lt %d\n', length(p.Results.CM));
                fprintf(fid,'%g %g %g 1\n', p.Results.CM');
                fprintf(fid,'\n');
            end

            fclose(fid);
        end
        
        % video files
        function wavi(dirname,filename,varargin)
            
            p = inputParser;
            addOptional(p,'FrameRate',24);
            addOptional(p,'Quality',100);
            addOptional(p,'ext','png');
            parse(p,varargin{:});
            
            outputVideo = VideoWriter( [dirname filename(1:end-1) '.avi'] );
            
            outputVideo.FrameRate = p.Results.FrameRate;
            outputVideo.Quality = p.Results.Quality;

            open(outputVideo);
            
            D = dir([dirname filename '*.' p.Results.ext]);
            N = length(D);
            for i = 1:N
                img = imread( [dirname D(i).name], ...
                              'BackgroundColor', [1,1,1,] );
                writeVideo(outputVideo,img);
            end
            
            close(outputVideo);
        end
        
        function wmpeg4(dirname,filename,varargin)
            p = inputParser;
            addOptional(p,'FrameRate',24);
            addOptional(p,'Quality',100);
            addOptional(p,'ext','png');
            parse(p,varargin{:});
            
            outputVideo = VideoWriter( [dirname filename(1:end-1) '.mp4'], ...
                                       'MPEG-4' );
            
            outputVideo.FrameRate = p.Results.FrameRate;
            outputVideo.Quality = p.Results.Quality;

            open(outputVideo);
            
            D = dir([dirname filename '*.' p.Results.ext]);
            N = length(D);
            for i = 1:N
                img = imread( [dirname D(i).name],'BackgroundColor','none');
                
                % manual padding to a multiple of 2
                sz = size(img);
                if mod(sz(1),2) == 1; img = [img; 255*ones(1,sz(2),3)]; end;
                
                sz = size(img);
                if mod(sz(2),2) == 1; img = [img 255*ones(sz(1),1,3)]; end;
                
                writeVideo(outputVideo,img);
            end
            
            close(outputVideo);
        end
       
        function montage( filename, imgnames )
            rows = 1; cols = 4;

            A = [];
            for i = 1:rows
                b = [];
                for j = 1:cols
                    a = imread( [imgnames{(i-1)*rows+j} '.png'] );
                    b = cat(2,b,a);
                end
                A = cat(1,A,b);
            end
            imwrite(A,[filename '.png']);
            delete( sprintf('%s_*.png',filename) );
            close all;
        end
    end
    
end

