classdef MESH_VIS
    %MESH_VIS Summary of this class goes here
    %   Detailed explanation goes here
    % TODO: mesh,vf
    
    properties
    end
    
    methods (Static)
        
        function h = mesh(M,varargin)
            p = inputParser;
            addOptional(p,'FaceColor','w');
            addOptional(p,'FaceAlpha',.9);
            addOptional(p,'EdgeColor','k');
            addOptional(p,'CW',[]);
            addOptional(p,'UVt',[]);
            addOptional(p,'FUVt',[]);
            addOptional(p,'Texture','cross.png');
            addOptional(p,'SingularPts',0);
            addOptional(p,'SingularPtsR',20);
            addOptional(p,'LandmarkPts',[]);
            addOptional(p,'LandmarkPtsRf',.01);
            addOptional(p,'LandmarkPtsC',[]);
            parse(p,varargin{:});
            
            if isempty( p.Results.UVt )
                if any(strcmp(properties(M), 'triangles')) == 1
                    h = patch('faces',M.triangles,'vertices',M.vertices, ...
                              'FaceColor',p.Results.FaceColor, ...
                              'EdgeColor',p.Results.EdgeColor, ...
                              'FaceAlpha',p.Results.FaceAlpha);
                else
                    if isempty(p.Results.CW) == 1
                        h = patch('faces',M.quads,'vertices',M.vertices, ...
                                  'FaceColor',p.Results.FaceColor, ...
                                  'EdgeColor',p.Results.EdgeColor, ...
                                  'FaceAlpha',p.Results.FaceAlpha);
                    else
                        
                        CW = 'CData';
                        if size(p.Results.CW,2) == 3; CW = 'FaceVertexCData'; end;
                        
                       h = patch('faces',M.quads,'vertices',M.vertices, ...
                                  CW,p.Results.CW,...
                                 'FaceColor','interp', ...
                                 'EdgeColor',p.Results.EdgeColor, ...
                                 'FaceAlpha',p.Results.FaceAlpha);
                    end
                    
                    if p.Results.SingularPts == 1
                        MESH_VIS.singular_pts( M, p.Results.SingularPtsR );
                        %                         camlight; lighting phong; material dull;
                    end
                    
                    if ~isempty(p.Results.LandmarkPts)   
                        hold on
                        MESH_VIS.landmarks_sph(M, p.Results.LandmarkPts, ...
                            p.Results.LandmarkPtsRf, p.Results.LandmarkPtsC)
                    end
                end
            else
                I = imread(p.Results.Texture);
                
                patcht2(M.triangles,M.vertices,...
                    p.Results.FUVt,p.Results.UVt,I);
            end
            
            cameratoolbar; cameratoolbar('SetCoordSys','none');
            axis equal; axis off;
            set(gcf,'windowstyle','docked');
        end
        
        function h = texture(M, varargin)
            p = inputParser;
            addOptional(p,'FaceColor','w');
            addOptional(p,'EdgeColor','k');
            parse(p,varargin{:});

            h = patch('vertices', M.uv, 'faces',M.quads_uv,...
                            'FaceColor', p.Results.FaceColor, ...
                            'EdgeColor', p.Results.EdgeColor);
                        
            cameratoolbar; cameratoolbar('SetCoordSys','none');
            axis equal; axis off;
            set(gcf,'windowstyle','docked');
        end
        
        function [ M2 ] = nc_mesh(M)
            M2.vertices = ( M.vertices(M.edges(:,1),:) + ...
                M.vertices(M.edges(:,2),:) )/2;
            M2.triangles = abs(M.t2e);
            
            M2.nf = M.nf;
            M2.nv = M.ne;
            
            M2.om = M;
            M2.om.vertices = M.vertices-M.Nv*M.mel/10;
        end
        
        function h = func(M,f,varargin)
            p = inputParser;
            addOptional(p,'EdgeColor','none');
            addOptional(p,'Caxis','auto');
            addOptional(p,'View',[0 1 0]);
            addOptional(p,'Dock',0);
            addOptional(p,'Colormap','jet');
            addOptional(p,'OpenGL',0);
            addOptional(p,'LightPos',[0 0 1]);
            parse(p,varargin{:});
            
            szf = size(f);
            if szf(1) == szf(2) && norm(f - diag(diag(f)),'fro') < 1e-8
                f = diag(f);
            end
            
            FC = 'w';
            if szf(1) == M.nv; FC = 'interp';
            elseif szf(1) == M.nv_uv; FC = 'interp';
            elseif szf(1) == M.nf; FC = 'flat';
            else
                M = MESH_VIS.nc_mesh( M );
                patch('faces',M.om.triangles,'vertices',M.vertices, ...
                    'FaceColor',FC, ...
                    'EdgeColor',p.Results.EdgeColor);
                FC = 'interp';
            end;
            
            CW = 'CData';
            if szf(2) == 3; CW = 'FaceVertexCData'; end;
            
            if any(strcmp(properties(M), 'triangles')) == 1
                patch('faces',M.triangles,'vertices',M.vertices, ...
                    CW,f, ...
                    'FaceColor',FC, ...
                    'EdgeColor',p.Results.EdgeColor);
            else
                if szf(1) == M.nv_uv; 
                    vts = M.uv; qds = M.quads_uv;
                else
                    vts = M.vertices; qds = M.quads;
                end
                h = patch('faces',qds,'vertices',vts, ...
                    CW,f, ...
                    'FaceColor',FC, ...
                    'EdgeColor',p.Results.EdgeColor);
            end
            
            cameratoolbar; cameratoolbar('SetCoordSys','none');
            axis equal; axis off;
            set(gcf,'windowstyle','docked');
            colorbar; caxis(p.Results.Caxis);
            
            if strcmp(p.Results.Colormap, 'none')
            else
                colormap(p.Results.Colormap);
            end
            
            if p.Results.OpenGL == 1
                camlight('headlight');
                lighting phong;
                material dull;
                %                 light('Position',p.Results.LightPos,'Style','infinite');
            end
            
            if p.Results.Dock == 1; set(gcf, 'WindowStyle', 'docked'); end;
        end
        
        function vf(M,vf,varargin)
            vf = reshape(vf,[],3);
            
            p = inputParser;
            addOptional(p,'FaceColor','w');
            addOptional(p,'EdgeColor','k');
            addOptional(p,'f',MESHQ.normv(vf));
            addOptional(p,'Color','k');
            addOptional(p,'Dock',0);
            addOptional(p,'nRosy',1);
            parse(p,varargin{:});
            
            hold on;
            if isempty(p.Results.f) == 1
                MESH_VIS.mesh(M, p);
            else
                MESH_VIS.func(M,p.Results.f);
            end
            
            if length(vf) == M.nf
                X = M.vertices;
                if any(strcmp(properties(M), 'triangles')) == 1
                    T = M.triangles;
                    Xm = (X(T(:,1),:)+X(T(:,2),:)+X(T(:,3),:))/3;
                else
                    Q = M.quads;
                    Xm = (X(Q(:,1),:)+X(Q(:,2),:)+X(Q(:,3),:)+X(Q(:,4),:))/4;
                end
            elseif length(vf) == M.ne
                X = M.vertices; E = M.edges;
                Xm = X(E(:,1),:)+X(E(:,2),:)/2;
            end
            
            scale = .5;
            R = speye(3*M.nf,3*M.nf);
            if (p.Results.nRosy > 1)
                for i = 1:(4/p.Results.nRosy); R = M.R*R; end;
            end
            
            % works only for n=2/4
            for i = 1:p.Results.nRosy
                quiver3( Xm(:,1), Xm(:,2), Xm(:,3), ...
                    vf(:,1), vf(:,2), vf(:,3), ...
                    scale, 'Color', p.Results.Color );
                
                vf = reshape(R*vf(:),[],3);
            end
            
            cameratoolbar; cameratoolbar('SetCoordSys','none');
            axis equal; axis off;
            set(gcf,'windowstyle','docked');
            colormap(jet);
            if p.Results.Dock == 1; set(gcf, 'WindowStyle', 'docked'); end;
            
            %             view([0 1 0]);
            
            hold off;
        end
        
        function set_camera(ca,cam)
            set(ca, 'PlotBoxAspectRatio',cam.pba);
            set(ca, 'DataAspectRatio',cam.dar);
            set(ca, 'CameraViewAngle',cam.cva);
            set(ca, 'CameraUpVector',cam.cuv);
            set(ca, 'CameraTarget',cam.ct);
            set(ca, 'CameraPosition',cam.cp);
        end
        
        function cam = get_camera(ca)
            cam.pba = get(ca, 'PlotBoxAspectRatio');
            cam.dar = get(ca, 'DataAspectRatio');
            cam.cva = get(ca, 'CameraViewAngle');
            cam.cuv = get(ca, 'CameraUpVector');
            cam.ct = get(ca, 'CameraTarget');
            cam.cp = get(ca, 'CameraPosition');
        end
        
        function singular_pts( M, r )
            X = M.vertices;
            
            SI3 = find(M.valence == 3);
            SI5 = find(M.valence == 5);
            
            [SX,SY,SZ] = sphere;
            a = max(max(X)-min(X))*r;
            SX = SX/a; SY = SY/a; SZ = SZ/a;
            
            hold on;
            for i = 1:numel(SI3)
                xs = X(SI3(i),:);
                surf(SX+xs(1),SY+xs(2),SZ+xs(3),'FaceColor','r',...
                    'EdgeColor','none','FaceAlpha',1);
            end
            
            for i = 1:numel(SI5)
                xs = X(SI5(i),:);
                surf(SX+xs(1),SY+xs(2),SZ+xs(3),'FaceColor','b',...
                    'EdgeColor','none','FaceAlpha',1);
            end
            hold off;
        end
        
        function cm2 = interp_cm( cm, sz )
            n = numel(cm(:,1));
            xq = linspace(0,1,sz);
            cm2 = zeros(sz,3);
            for i = 1:3
                cm2(:,i) = interp1(cm(:,1),cm(:,i+1),xq);
            end
        end
        
        function landmarks(M, landmarks, edge_color)
            % visualize landmarks - delta on vertices
            f = zeros(M.nv, 1);
            
            for i = 1:numel(landmarks)
                a0 = 500; % STD for the src
                dist0 = sqrt(sum(bsxfun(@minus,M.vertices,M.vertices(landmarks(i),:)).^2,2));
                rho0 = exp(-dist0.^2*a0);
                rho0 = rho0/sum(rho0);
                f = f + rho0;
            end
            
            MESH_VIS.func(M, f, 'EdgeColor', edge_color);
            
        end
        
        function h = landmarks_fine(M, landmarks_fine, S_to_fine, r, C)
            if nargin < 4
                r = 100;
            end
            if nargin < 5
                C = cbrewer('qual', 'Set1', length(landmarks_fine));
            end
            
            %vis mesh
            h = MESH_VIS.mesh(M);
            hold on;
            
            % scatter fine landmarks
            X = S_to_fine*M.vertices;
            X = X(landmarks_fine,:);
            scatter3sph(X(:,1), X(:,2), X(:,3),'size',r,'transp',1,'color', C);
        end
        
        function landmarks_sph(M, landmarks_fine, rf, C)
            if nargin < 4
                rf = 0.01;
            end
            if nargin < 5 || isempty(C)
                C = cbrewer('qual', 'Set1', length(landmarks_fine));
            end
            
            % scatter fine landmarks
            X = M.vertices;
            X = X(landmarks_fine,:);
            r = rf*(max(max(M.vertices))-min(min(M.vertices)));
            scatter3sph(X(:,1), X(:,2), X(:,3),'size',r,'transp',1,'color', C);
        end
        
        function suffix()
            cameratoolbar; cameratoolbar('SetCoordSys','none');
            axis equal; axis off;
            set(gcf,'windowstyle','docked');        
        end
    end
    
end

