#include "mex.h"
#include <vector>
#include <math.h>

using namespace std;

#define PI 3.14159265

class Halfedge{
public:
    Halfedge* next;
    Halfedge* prev;
    int from;
    Halfedge* twin;
    int edge;
    int face;
    int inface;
    
    Halfedge():next(NULL), from(-1),prev(NULL), twin(NULL){}
    ~Halfedge(){}
};

class Vector{
public:
    double x,y,z;
    Vector(double xx, double yy, double zz):x(xx), y(yy), z(zz){}
    Vector():x(0.0),y(0.0), z(0.0){}
    ~Vector(){}
    
    Vector operator+(Vector v){return Vector(x+v.x,y+v.y,z+v.z);}
    Vector operator-(Vector v){return Vector(x-v.x,y-v.y,z-v.z);}
    Vector operator*(double m){return Vector(x*m, y*m, z*m);}
};

void GetVirtualPoints(Vector& vpoint,Halfedge* FromEdge, int n, double* X, int XSize)
{
    int h=FromEdge->next->twin->next->next->from;
    int l=FromEdge->next->from;
    int lm1=FromEdge->twin->next->next->from;
    int lp1=FromEdge->prev->from;
    int k=FromEdge->next->next->from;
    int km1=FromEdge->twin->prev->from;
    int kp1=FromEdge->prev->twin->next->next->from;
    int km2=FromEdge->twin->next->twin->prev->from;
    Vector suml, sumk;
    Halfedge* ebegin=FromEdge;
    Halfedge* ecurr=ebegin;
    do{
        suml=suml+Vector(X[ecurr->next->from], X[ecurr->next->from+XSize], X[ecurr->next->from+2*XSize]);
        sumk=sumk+Vector(X[ecurr->next->next->from], X[ecurr->next->next->from+XSize], X[ecurr->next->next->from+2*XSize]);
        ecurr=ecurr->twin->next;
    }while (ecurr!=ebegin);
    suml=suml*(4.0/n);
    sumk=sumk*(4.0/(5.0*n));
    Vector ljpm=Vector(X[l],X[l+XSize],X[l+2*XSize])+Vector(X[lm1],X[lm1+XSize],X[lm1+2*XSize])+Vector(X[lp1],X[lp1+XSize],X[lp1+2*XSize]);
    Vector ks=(Vector(X[k],X[k+XSize],X[k+2*XSize])+Vector(X[km1],X[km1+XSize],X[km1+2*XSize])+Vector(X[km2],X[km2+XSize],X[km2+2*XSize])+Vector(X[kp1],X[kp1+XSize],X[kp1+2*XSize]))*(1.0/5.0);
    vpoint=suml-sumk-ljpm+ks;         
}
        
        
        
void mexFunction(
    int nlhs, mxArray *plhs[],
    int nrhs, const mxArray *prhs[])
{
   /* more C/C++ code ... */
    
    double* X=(double*)mxGetPr(prhs[0]);
    double* Q=(double*)mxGetPr(prhs[1]);
   // nlhs=4;
    
    int XSize=mxGetM(prhs[0]);
    int QSize=mxGetM(prhs[1]);
    vector<Halfedge*> Halfedges(QSize*4);
    int EdgeIndex=0;
    
    for (int i=0;i<QSize;i++){
        Halfedge* ebegin=NULL;
        Halfedge* eprev=NULL;
        for (int j=0;j<4;j++){
            Halfedge* ecurr=new Halfedge;
            ecurr->from=Q[QSize*j+i]-1;
            ecurr->face=i;
            ecurr->inface=j;
            if (eprev!=NULL){
                ecurr->prev=eprev;
                eprev->next=ecurr;
            }
            if (ebegin==NULL)
                ebegin=ecurr;
            eprev=ecurr;
            Halfedges[QSize*j+i]=ecurr;
        }
        ebegin->prev=eprev;
        eprev->next=ebegin;
    }
    
    //finding twins
    for (int i=0;i<Halfedges.size();i++)
        for (int j=i+1;j<Halfedges.size();j++)
            if ((Halfedges[i]->next->from==Halfedges[j]->from)&&(Halfedges[j]->next->from==Halfedges[i]->from)){
                Halfedges[i]->twin=Halfedges[j];
                Halfedges[j]->twin=Halfedges[i];
                Halfedges[i]->edge=Halfedges[j]->edge=EdgeIndex++;
            }

	for (int i=0;i<Halfedges.size();i++)
		if (Halfedges[i]->twin==NULL)
			Halfedges[i]->edge=EdgeIndex++;
    
    
    //computing valences
    vector<int> Valences(XSize);
    vector<bool> BoundaryVertex(XSize);
    for (int i=0;i<XSize;i++){
        Valences[i]=0; BoundaryVertex[i]=false;
    }
    for (int i=0;i<Halfedges.size();i++){
        Valences[Halfedges[i]->from]++;
        if (Halfedges[i]->twin==NULL){  //have to supplement for missing twin
            Valences[Halfedges[i]->next->from]++;
            BoundaryVertex[Halfedges[i]->from]=true;
        }
    }

	//computing face points
	vector<Vector> FacePoints(QSize);
	for (int i=0;i<Halfedges.size();i++)
	{
		Vector x=Vector(X[Halfedges[i]->from], X[Halfedges[i]->from+XSize], X[Halfedges[i]->from+2*XSize]);
		FacePoints[Halfedges[i]->face]=FacePoints[Halfedges[i]->face]+x*0.25;
	}

	//computing edge points
	vector<Vector> EdgePoints(EdgeIndex);
	for (int i=0;i<Halfedges.size();i++){
		if ((Halfedges[i]->next->from < Halfedges[i]->from)&&(Halfedges[i]->twin!=NULL))
            continue;  //subdominant side

		Vector xfrom=Vector(X[Halfedges[i]->from], X[Halfedges[i]->from+XSize], X[Halfedges[i]->from+2*XSize]);
        Vector xto=Vector(X[Halfedges[i]->next->from], X[Halfedges[i]->next->from+XSize], X[Halfedges[i]->next->from+2*XSize]);

		if (Halfedges[i]->twin==NULL){  //boundary edge
			EdgePoints[Halfedges[i]->edge]=(xfrom+xto)*0.5;
			continue;
		}

		//all other edges are inner, and the only thing that varies is the weight of the boundary vertex within

		int v1=Halfedges[i]->prev->from;
		int v2=Halfedges[i]->next->next->from;
		int v3=Halfedges[i]->twin->prev->from;
		int v4=Halfedges[i]->twin->next->next->from;
		Vector x1=Vector(X[v1], X[v1+XSize], X[v1+2*XSize]);
		Vector x2=Vector(X[v2], X[v2+XSize], X[v2+2*XSize]);
		Vector x3=Vector(X[v3], X[v3+XSize], X[v3+2*XSize]);
		Vector x4=Vector(X[v4], X[v4+XSize], X[v4+2*XSize]);
		double wfrom, wto;
		/*if (BoundaryVertex[Halfedges[i]->from]){
			double lambda=(3.0/8.0)-(1.0/4.0)*cos(PI/((double)Valences[Halfedges[i]->from]));
			wfrom=lambda;
			wto=3.0/4.0-lambda;
		} else if (BoundaryVertex[Halfedges[i]->next->from]){
			double lambda=(3.0/8.0)-(1.0/4.0)*cos(PI/((double)Valences[Halfedges[i]->next->from]));
			wto=lambda;
			wfrom=3.0/4.0-lambda;
		} else{  //totally inner edge*/
			wto=wfrom=3.0/8.0;
		//}
		EdgePoints[Halfedges[i]->edge]=xfrom*wfrom+xto*wto+(x1+x2+x3+x4)*(1.0/16.0);
	}

	//computing vertex points
	vector<Vector> VertexPoints(XSize);
	vector<bool> VertexDone(XSize);
	for (int i=0;i<XSize;i++)
		VertexDone[i]=false;

	//dealing with the easier case of bound vertices by halfedge iterating
	for (int i=0;i<Halfedges.size();i++){
		if (Halfedges[i]->twin!=NULL)
			continue;
		Vector xfrom=Vector(X[Halfedges[i]->from], X[Halfedges[i]->from+XSize], X[Halfedges[i]->from+2*XSize]);
        Vector xto=Vector(X[Halfedges[i]->next->from], X[Halfedges[i]->next->from+XSize], X[Halfedges[i]->next->from+2*XSize]);
		VertexPoints[Halfedges[i]->from]=VertexPoints[Halfedges[i]->from]+xfrom*(3.0/8.0)+xto*(1.0/8.0);
		VertexPoints[Halfedges[i]->next->from]=VertexPoints[Halfedges[i]->next->from]+xto*(3.0/8.0)+xfrom*(1.0/8.0);
	}
	for (int i=0;i<Halfedges.size();i++){
		if ((VertexDone[Halfedges[i]->from])||(BoundaryVertex[Halfedges[i]->from]))
			continue;
		int CurrVertex=Halfedges[i]->from;
		double beta=3.0/(2.0*(double)Valences[CurrVertex]);
		double delta=1.0/(4.0*(double)Valences[CurrVertex]);
		Halfedge* ebegin=Halfedges[i];
		Halfedge* eiterate=ebegin;
		do{
			Vector xto=Vector(X[eiterate->next->from], X[eiterate->next->from+XSize], X[eiterate->next->from+2*XSize]);
			Vector xacross=Vector(X[eiterate->next->next->from], X[eiterate->next->next->from+XSize], X[eiterate->next->next->from+2*XSize]);
			VertexPoints[CurrVertex]=VertexPoints[CurrVertex]+xto*(beta/((double)Valences[CurrVertex]));
			VertexPoints[CurrVertex]=VertexPoints[CurrVertex]+xacross*(delta/((double)Valences[CurrVertex]));
			eiterate=eiterate->twin->next;
		}while (eiterate!=ebegin);

		Vector xfrom=Vector(X[eiterate->from], X[eiterate->from+XSize], X[eiterate->from+2*XSize]);
		VertexPoints[CurrVertex]=VertexPoints[CurrVertex]+xfrom*(1.0-beta-delta);
		VertexDone[CurrVertex]=true;
	}

   
    //creating edge lists
    mxArray *Elist;
	double *vpMatrix;
    double *epMatrix;
    double* elistMatrix;
    double* e2tMatrix;
    double* t2eMatrix;
	double* fpMatrix;
    mwSize dims[2];
    dims[0]=EdgeIndex;
    dims[1]=2;
    
    /* create the output matrices */
	plhs[0] = mxCreateDoubleMatrix(XSize,3,mxREAL);
    vpMatrix=(double*)mxGetPr(plhs[0]);
    plhs[1] = mxCreateDoubleMatrix(EdgeIndex,3,mxREAL);
    epMatrix=(double*)mxGetPr(plhs[1]);
    plhs[2] = mxCreateDoubleMatrix(EdgeIndex,2,mxREAL);
    elistMatrix=(double*)mxGetPr(plhs[2]);
    plhs[3] = mxCreateDoubleMatrix(EdgeIndex,2, mxREAL);
    e2tMatrix=(double*)mxGetPr(plhs[3]);
    dims[0]=QSize;
    dims[1]=4;
    plhs[4] = mxCreateDoubleMatrix(QSize,4,mxREAL);
    t2eMatrix=(double*)mxGetPr(plhs[4]);
	plhs[5] = mxCreateDoubleMatrix(QSize,3,mxREAL);
    fpMatrix=(double*)mxGetPr(plhs[5]);
    for (int i=0;i<Halfedges.size();i++){
          if ((Halfedges[i]->next->from < Halfedges[i]->from)&&(Halfedges[i]->twin!=NULL))
            continue; 
          epMatrix[Halfedges[i]->edge]=EdgePoints[Halfedges[i]->edge].x;
          epMatrix[Halfedges[i]->edge+EdgeIndex]=EdgePoints[Halfedges[i]->edge].y;
          epMatrix[Halfedges[i]->edge+2*EdgeIndex]=EdgePoints[Halfedges[i]->edge].z;
          elistMatrix[Halfedges[i]->edge]=Halfedges[i]->from+1;
          elistMatrix[Halfedges[i]->edge+EdgeIndex]=Halfedges[i]->next->from+1;
          e2tMatrix[Halfedges[i]->edge]=Halfedges[i]->face+1;
          if (Halfedges[i]->twin!=NULL)
              e2tMatrix[Halfedges[i]->edge+EdgeIndex]=-(Halfedges[i]->twin->face+1);
          else
              e2tMatrix[Halfedges[i]->edge+EdgeIndex]=0;
		  if ((Halfedges[i]->face+Halfedges[i]->inface*QSize<0)||(Halfedges[i]->face+Halfedges[i]->inface*QSize>4*QSize-1))
			  int kaka=9;
		  //continue;
          t2eMatrix[Halfedges[i]->face+Halfedges[i]->inface*QSize]=Halfedges[i]->edge+1;
		  if (Halfedges[i]->twin!=NULL)
			  t2eMatrix[Halfedges[i]->twin->face+Halfedges[i]->twin->inface*QSize]=Halfedges[i]->edge+1;
    }
	for (int i=0;i<QSize;i++){
		fpMatrix[i]=FacePoints[i].x;
		fpMatrix[i+QSize]=FacePoints[i].y;
		fpMatrix[i+2*QSize]=FacePoints[i].z;
	}
	for (int i=0;i<XSize;i++){
		vpMatrix[i]=VertexPoints[i].x;
		vpMatrix[i+XSize]=VertexPoints[i].y;
		vpMatrix[i+2*XSize]=VertexPoints[i].z;
	}

}
