#include "mex.h"
#include "./eigen/Eigen/Core"
#include <vector>
#include <math.h>
#include <iostream>

using namespace std;
using namespace Eigen;

#define PI 3.14159265

/*class Halfedge{
 * public:
 * Halfedge* next;
 * Halfedge* prev;
 * int from;
 * Halfedge* twin;
 * int edge;
 * int face;
 * int inface;
 *
 * Halfedge():next(NULL), from(-1),prev(NULL), twin(NULL){}
 * ~Halfedge(){}
 * };*/

/*class Vector{
 * public:
 * double x,y,z;
 * Vector(double xx, double yy, double zz):x(xx), y(yy), z(zz){}
 * Vector():x(0.0),y(0.0), z(0.0){}
 * ~Vector(){}
 *
 * Vector operator+(Vector v){return Vector(x+v.x,y+v.y,z+v.z);}
 * Vector operator-(Vector v){return Vector(x-v.x,y-v.y,z-v.z);}
 * Vector operator*(double m){return Vector(x*m, y*m, z*m);}
 * };*/

/*void GetVirtualPoints(Vector& vpoint,Halfedge* FromEdge, int n, double* X, int XSize)
 * {
 * int h=FromEdge->next->twin->next->next->from;
 * int l=FromEdge->next->from;
 * int lm1=FromEdge->twin->next->next->from;
 * int lp1=FromEdge->prev->from;
 * int k=FromEdge->next->next->from;
 * int km1=FromEdge->twin->prev->from;
 * int kp1=FromEdge->prev->twin->next->next->from;
 * int km2=FromEdge->twin->next->twin->prev->from;
 * Vector suml, sumk;
 * Halfedge* ebegin=FromEdge;
 * Halfedge* ecurr=ebegin;
 * do{
 * suml=suml+Vector(X[ecurr->next->from], X[ecurr->next->from+XSize], X[ecurr->next->from+2*XSize]);
 * sumk=sumk+Vector(X[ecurr->next->next->from], X[ecurr->next->next->from+XSize], X[ecurr->next->next->from+2*XSize]);
 * ecurr=ecurr->twin->next;
 * }while (ecurr!=ebegin);
 * suml=suml*(4.0/n);
 * sumk=sumk*(4.0/(5.0*n));
 * Vector ljpm=Vector(X[l],X[l+XSize],X[l+2*XSize])+Vector(X[lm1],X[lm1+XSize],X[lm1+2*XSize])+Vector(X[lp1],X[lp1+XSize],X[lp1+2*XSize]);
 * Vector ks=(Vector(X[k],X[k+XSize],X[k+2*XSize])+Vector(X[km1],X[km1+XSize],X[km1+2*XSize])+Vector(X[km2],X[km2+XSize],X[km2+2*XSize])+Vector(X[kp1],X[kp1+XSize],X[kp1+2*XSize]))*(1.0/5.0);
 * vpoint=suml-sumk-ljpm+ks;
 * }*/

void polygonal_edge_topology(const Eigen::VectorXi& D,
        const Eigen::MatrixXi& F,
        Eigen::MatrixXi& EV,
        Eigen::MatrixXi& FE,
        Eigen::MatrixXi& EF,
        Eigen::MatrixXi& EFi,
        Eigen::MatrixXd& FEs,
        Eigen::VectorXi& InnerEdges)
{
    // Only needs to be edge-manifold
    std::vector<std::vector<int> > ETT;
    for(int f=0;f<D.rows();++f)
        for (int i=0;i<D(f);++i)
        {
            // v1 v2 f vi
            int v1 = F(f,i);
            int v2 = F(f,(i+1)%D(f));
            if (v1 > v2) std::swap(v1,v2);
            std::vector<int> r(4);
            r[0] = v1; r[1] = v2;
            r[2] = f;  r[3] = i;
            ETT.push_back(r);
        }
    std::sort(ETT.begin(),ETT.end());
    
    // count the number of edges (assume manifoldness)
    int En = 1; // the last is always counted
    for(unsigned i=0;i<ETT.size()-1;++i)
        if (!((ETT[i][0] == ETT[i+1][0]) && (ETT[i][1] == ETT[i+1][1])))
            ++En;
    
    EV = Eigen::MatrixXi::Constant((int)(En),2,-1);
    FE = Eigen::MatrixXi::Constant((int)(F.rows()),(int)(F.cols()),-1);
    EF = Eigen::MatrixXi::Constant((int)(En),2,-1);
    En = 0;
    
    for(unsigned i=0;i<ETT.size();++i)
    {
        if (i == ETT.size()-1 ||
                !((ETT[i][0] == ETT[i+1][0]) && (ETT[i][1] == ETT[i+1][1]))
                )
        {
            // Border edge
            std::vector<int>& r1 = ETT[i];
            EV(En,0)     = r1[0];
            EV(En,1)     = r1[1];
            EF(En,0)    = r1[2];
            FE(r1[2],r1[3]) = En;
        }
        else
        {
            std::vector<int>& r1 = ETT[i];
            std::vector<int>& r2 = ETT[i+1];
            EV(En,0)     = r1[0];
            EV(En,1)     = r1[1];
            EF(En,0)    = r1[2];
            EF(En,1)    = r2[2];
            FE(r1[2],r1[3]) = En;
            FE(r2[2],r2[3]) = En;
            ++i; // skip the next one
        }
        ++En;
    }
    
    // Sort the relation EF, accordingly to EV
    // the first one is the face on the left of the edge
    
    for(unsigned i=0; i<EF.rows(); ++i)
    {
        int fid = EF(i,0);
        bool flip = true;
        // search for edge EV.row(i)
        for (unsigned j=0; j<D(fid); ++j)
        {
            if ((F(fid,j) == EV(i,0)) && (F(fid,(j+1)%D(fid)) == EV(i,1)))
                flip = false;
        }
        
        if (flip)
        {
            int tmp = EF(i,0);
            EF(i,0) = EF(i,1);
            EF(i,1) = tmp;
        }
    }
    
    
    std::vector<int> InnerEdgesVec;
    EFi=Eigen::MatrixXi::Constant(EF.rows(), 2,-1);
    FEs=Eigen::MatrixXd::Zero(FE.rows(),FE.cols());
    for (int i=0;i<EF.rows();i++)
        for (int k=0;k<2;k++){
            if (EF(i,k)==-1)
                continue;
            
            for (int j=0;j<D(EF(i,k));j++)
                if (FE(EF(i,k),j)==i)
                    EFi(i,k)=j;
        }
    
    for (int i=0;i<EF.rows();i++){
        if (EFi(i,0)!=-1) FEs(EF(i,0),EFi(i,0))=1.0;
        if (EFi(i,1)!=-1) FEs(EF(i,1),EFi(i,1))=-1.0;
        if ((EF(i,0)!=-1)&&(EF(i,1)!=-1))
            InnerEdgesVec.push_back(i);
    }
    
    InnerEdges.resize(InnerEdgesVec.size());
    for (int i=0;i<InnerEdgesVec.size();i++)
        InnerEdges(i)=InnerEdgesVec[i];
    
    
}


void dcel(const Eigen::MatrixXi& D,
        const Eigen::MatrixXi& F,
        const Eigen::MatrixXi& EV,
        const Eigen::MatrixXi& EF,
        const Eigen::MatrixXi& EFi,
        const Eigen::VectorXi& innerEdges,
        Eigen::VectorXi& VH,
        Eigen::MatrixXi& EH,
        Eigen::MatrixXi& FH,
        Eigen::VectorXi& HV,
        Eigen::VectorXi& HE,
        Eigen::VectorXi& HF,
        Eigen::VectorXi& nextH,
        Eigen::VectorXi& prevH,
        Eigen::VectorXi& twinH)
{
    //doing a local halfedge structure for polygonal meshes
    EH=Eigen::MatrixXi::Constant(EV.rows(),2,-1);
    int numH=0;
    
    for (int i=0;i<EF.rows();i++){
        if (EF(i,0)!=-1)
            EH(i,0)=numH++;
        if (EF(i,1)!=-1)
            EH(i,1)=numH++;
    }
    
    
    //halfedges to edge
    HE.conservativeResize(numH);
    for (int i=0;i<EH.rows();i++){
        if (EH(i,0)!=-1)
            HE(EH(i,0))=i;
        if (EH(i,1)!=-1)
            HE(EH(i,1))=i;
    }
    
    //halfedge to vertex and vice versa
    HV.conservativeResize(numH);
    VH.conservativeResize(EV.maxCoeff()+1);
    for (int i=0;i<EV.rows();i++){
        if (EH(i,0)!=-1){
            HV(EH(i,0))=EV(i,0);
            VH(EV(i,0))=EH(i,0);
        }
        if (EH(i,1)!=-1){
            HV(EH(i,1))=EV(i,1);
            VH(EV(i,1))=EH(i,1);
        }
    }
    
    //halfedge to twin
    twinH=Eigen::VectorXi::Constant(numH, -1);
    for (int i=0;i<EH.rows();i++)
        if ((EH(i,0)!=-1)&&(EH(i,1)!=-1)){
            twinH(EH(i,0))=EH(i,1);
            twinH(EH(i,1))=EH(i,0);
        }
    
    //faces to halfedges and vice versa
    FH.resize(F.rows(), F.cols());
    HF.resize(numH);
    for (int i=0;i<EF.rows();i++){
        if (EF(i,0)!=-1){
            FH(EF(i,0),EFi(i,0))=EH(i,0);
            HF(EH(i,0))=EF(i,0);
        }
        if (EF(i,1)!=-1){
            FH(EF(i,1),EFi(i,1))=EH(i,1);
            HF(EH(i,1))=EF(i,1);
        }
    }
    
    //halfedge to next and prev
    nextH.conservativeResize(HE.rows());
    prevH.conservativeResize(HE.rows());
    for (int i=0;i<D.rows();i++){
        for (int j=0;j<D(i);j++){
            nextH(FH(i,j))=FH(i,(j+1)%D(i));
            prevH(FH(i,(j+1)%D(i)))=FH(i,j);
        }
    }
    
}



void mexFunction(
        int nlhs, mxArray *plhs[],
        int nrhs, const mxArray *prhs[])
{
    /* more C/C++ code ... */
    
    double* X=(double*)mxGetPr(prhs[0]);
    double* Q=(double*)mxGetPr(prhs[1]);
    // nlhs=4;
    
    int XSize=mxGetM(prhs[0]);
    int QSize=mxGetM(prhs[1]);
    
    MatrixXi F,EV,EF, EFi;
    VectorXi innerEdges, VH;
    MatrixXi EH,FH, FE;
    VectorXi HV, HE, HF,nextH,prevH,twinH;
    MatrixXd FEs;
    
    MatrixXi QMat(QSize,4);
    for (int i=0;i<QSize;i++)
        for (int j=0;j<4;j++)
            QMat(i,j) = Q[QSize*j+i]-1;
    
    polygonal_edge_topology(MatrixXi::Constant(QSize,1,4),QMat,EV,FE,EF,EFi,FEs, innerEdges);
    
    dcel(MatrixXi::Constant(QSize,1,4),QMat, EV, EF,EFi,innerEdges, VH,EH,FH,HV,HE,HF,nextH, prevH,twinH);
    
    cout<<"EH: "<<EH<<endl;
    //vector<Halfedge*> Halfedges(QSize*4);

    vector<double> rowList, colList, valList;
    
    cout<<"computed polygonal_edge_topology and dcel"<<endl;
    
    /*for (int i=0;i<QSize;i++){
        Halfedge* ebegin=NULL;
        Halfedge* eprev=NULL;
        for (int j=0;j<4;j++){
            Halfedge* ecurr=new Halfedge;
            ecurr->from=Q[QSize*j+i]-1;
            ecurr->face=i;
            ecurr->inface=j;
            if (eprev!=NULL){
                ecurr->prev=eprev;
                eprev->next=ecurr;
            }
            if (ebegin==NULL)
                ebegin=ecurr;
            eprev=ecurr;
            Halfedges[QSize*j+i]=ecurr;
        }
        ebegin->prev=eprev;
        eprev->next=ebegin;
    }
    
    //finding twins
    for (int i=0;i<Halfedges.size();i++)
        for (int j=i+1;j<Halfedges.size();j++)
            if ((Halfedges[i]->next->from==Halfedges[j]->from)&&(Halfedges[j]->next->from==Halfedges[i]->from)){
                Halfedges[i]->twin=Halfedges[j];
                Halfedges[j]->twin=Halfedges[i];
                Halfedges[i]->edge=Halfedges[j]->edge=EdgeIndex++;
            }
    
    for (int i=0;i<Halfedges.size();i++)
        if (Halfedges[i]->twin==NULL)
            Halfedges[i]->edge=EdgeIndex++;*/
    
    
    //computing valences
    VectorXi Valences=VectorXi::Zero(XSize);
    VectorXi BoundaryVertex=VectorXi::Zero(XSize);
    for (int i=0;i<HV.rows();i++){
        Valences(HV(i))++;
        if (twinH(i)==-1){  //have to supplement for missing twin
            Valences(HV(nextH(i)))++;
            BoundaryVertex(HV(i))=true;
        }
    }
    
    cout<<"computed Valences"<<endl;
    
    //computing face points
    cout<<"QSize"<<QSize<<endl;
    MatrixXd FacePoints=MatrixXd::Zero(QSize,3);
    for (int i=0;i<HV.rows();i++)
    {
        RowVector3d x; x<<X[HV(i)], X[HV(i)+XSize], X[HV(i)+2*XSize];
        cout<<"HF(i): "<<HF(i)<<endl;
        FacePoints.row(HF(i))+=x*0.25;
        
        rowList.push_back(HF(i)+EV.rows()+XSize);
        colList.push_back(HV(i));
        valList.push_back(0.25);
    }
    
     cout<<"computed FacePoints"<<endl;
     cout<<"FacePoints: "<<endl;
    
    //computing edge points
    MatrixXd EdgePoints=MatrixXd::Zero(EH.rows(),3);
    
      //cout<<"EdgeIndex: "<<EdgeIndex<<endl;
    cout<<"EH.rows(): "<<EH.rows()<<endl;
    for (int i=0;i<EH.rows();i++){
        /*if ((Halfedges[i]->next->from < Halfedges[i]->from)&&(Halfedges[i]->twin!=NULL))
            continue;  //subdominant side*/
        
        int currH = (EH(i,0)==-1 ? EH(i,1) : EH(i,0));
        
         cout<<"currH: "<<currH<<endl;
        
        RowVector3d xfrom; xfrom<<X[HV(currH)], X[HV(currH)+XSize], X[HV(currH)+2*XSize];
        RowVector3d xto; xto<<X[HV(nextH(currH))], X[HV(nextH(currH))+XSize], X[HV(nextH(currH))+2*XSize];
        
        cout<<"HV(currH): "<<HV(currH)<<endl;
        cout<<"HV(nextH(currH)): "<<HV(nextH(currH))<<endl;
        
        if (1){  //boundary edge
            EdgePoints.row(i)=(xfrom+xto)*0.5;
            rowList.push_back(XSize+i);
            colList.push_back(HV(currH));
            valList.push_back(0.5);
            rowList.push_back(XSize+i);
            colList.push_back(HV(nextH(currH)));
            valList.push_back(0.5);
            continue;
        }
        
        //all other edges are inner, and the only thing that varies is the weight of the boundary vertex within
        
        int v1=HV(prevH(currH));        //Halfedges[i]->prev->from;
        int v2=HV(nextH(nextH(currH))); //Halfedges[i]->next->next->from;
        int v3=HV(prevH(twinH(currH))); //Halfedges[i]->twin->prev->from;
        int v4=HV(nextH(nextH(twinH(currH)))); //Halfedges[i]->twin->next->next->from;
        RowVector3d x1; x1<<X[v1], X[v1+XSize], X[v1+2*XSize];
        RowVector3d x2; x2<<X[v2], X[v2+XSize], X[v2+2*XSize];
        RowVector3d x3; x3<<X[v3], X[v3+XSize], X[v3+2*XSize];
        RowVector3d x4; x4<<X[v4], X[v4+XSize], X[v4+2*XSize];
        double wfrom, wto;
    
        wto=wfrom=3.0/8.0;
      
        EdgePoints.row(i)=xfrom*wfrom+xto*wto+(x1+x2+x3+x4)*(1.0/16.0);
        rowList.push_back(XSize+i);
        colList.push_back(HV(currH));
        valList.push_back(wfrom);
        rowList.push_back(XSize+i);
        colList.push_back(HV(nextH(currH)));
        valList.push_back(wto);
        
        rowList.push_back(XSize+i);
        colList.push_back(v1);
        valList.push_back(1.0/16.0);
        rowList.push_back(XSize+i);
        colList.push_back(v2);
        valList.push_back(1.0/16.0);
        rowList.push_back(XSize+i);
        colList.push_back(v3);
        valList.push_back(1.0/16.0);
        rowList.push_back(XSize+i);
        colList.push_back(v4);
        valList.push_back(1.0/16.0);
    }
    
    cout<<"computed EdgePoints"<<endl;
    
    //computing vertex points
    MatrixXd VertexPoints=MatrixXd::Zero(XSize,3);
    VectorXi VertexDone=VectorXi::Zero(XSize);
    
    for (int i=0;i<XSize;i++){
        VertexPoints.row(i)<<X[i], X[i+XSize], X[i+2*XSize];
        rowList.push_back(i);
        colList.push_back(i);
        valList.push_back(1.0);
    }
    
    //dealing with the easier case of bound vertices by halfedge iterating
    /*for (int i=0;i<HV.rows();i++){
        if (twinH(i)!=-1)
            continue;
        RowVector3d xfrom; xfrom<<X[HV(i)], X[HV(i)+XSize], X[HV(i)+2*XSize];
        RowVector3d xto; xto<<X[HV(nextH(i))], X[HV(nextH(i))+XSize], X[HV(nextH(i))+2*XSize];
        VertexPoints.row(HV(i))+=xfrom*(3.0/8.0)+xto*(1.0/8.0);
        VertexPoints.row(HV(nextH(i)))+=xto*(3.0/8.0)+xfrom*(1.0/8.0);
        rowList.push_back(HV(i));
        colList.push_back(HV(i));
        valList.push_back(3.0/8.0);
        rowList.push_back(HV(i));
        colList.push_back(HV(nextH(i)));
        valList.push_back(1.0/8.0);
        rowList.push_back(HV(nextH(i)));
        colList.push_back(HV(nextH(i)));
        valList.push_back(3.0/8.0);
        rowList.push_back(HV(nextH(i)));
        colList.push_back(HV(i));
        valList.push_back(1.0/8.0);
        
    }
    
    cout<<"computed Boundary Vertex Points"<<endl;
    
    for (int i=0;i<HV.rows();i++){
        if ((VertexDone(HV(i)))||(BoundaryVertex(HV(i))))
            continue;
        int CurrVertex=HV(i);
        double beta=3.0/(2.0*(double)Valences(CurrVertex));
        double delta=1.0/(4.0*(double)Valences(CurrVertex));
        int ebegin=i;
        int eiterate=ebegin;
        do{
            RowVector3d xto; xto<<X[HV(nextH(eiterate))], X[HV(nextH(eiterate))+XSize], X[HV(nextH(eiterate))+2*XSize];
            RowVector3d xacross; xacross<<X[HV(nextH(nextH(eiterate)))], X[HV(nextH(nextH(eiterate)))+XSize], X[HV(nextH(nextH(eiterate)))+2*XSize];
            VertexPoints.row(CurrVertex)+=xto*(beta/((double)Valences(CurrVertex)));
            VertexPoints.row(CurrVertex)+=xacross*(delta/((double)Valences(CurrVertex)));
            
            rowList.push_back(CurrVertex);
            colList.push_back(HV(nextH(eiterate)));
            valList.push_back(beta/((double)Valences(CurrVertex)));
            
            rowList.push_back(CurrVertex);
            colList.push_back(HV(nextH(nextH(eiterate))));
            valList.push_back(delta/((double)Valences(CurrVertex)));
            
            eiterate=nextH(twinH(eiterate));
        }while (eiterate!=ebegin);
        
        RowVector3d xfrom; xfrom<<X[HV(eiterate)], X[HV(eiterate)+XSize], X[HV(eiterate)+2*XSize];
        VertexPoints.row(CurrVertex)+=xfrom*(1.0-beta-delta);
        
        rowList.push_back(CurrVertex);
        colList.push_back(HV(eiterate));
        valList.push_back(1.0-beta-delta);
        
        VertexDone(CurrVertex)=true;
    }*/
    
    //cout<<"computed inner Vertex Points"<<endl;
    
    
    //creating edge lists
    mxArray *Elist;
    double *vpMatrix;
    double *epMatrix;
    double* elistMatrix;
    double* e2tMatrix;
    double* t2eMatrix;
    double* fpMatrix;
    double *rowVector, *colVector, *valVector;
    mwSize dims[2];
    dims[0]=EV.rows();
    dims[1]=2;
    
    /* create the output matrices */
    plhs[0] = mxCreateDoubleMatrix(XSize,3,mxREAL);
    vpMatrix=(double*)mxGetPr(plhs[0]);
    plhs[1] = mxCreateDoubleMatrix(EV.rows(),3,mxREAL);
    epMatrix=(double*)mxGetPr(plhs[1]);
    plhs[2] = mxCreateDoubleMatrix(EV.rows(),2,mxREAL);
    elistMatrix=(double*)mxGetPr(plhs[2]);
    plhs[3] = mxCreateDoubleMatrix(EV.rows(),2, mxREAL);
    e2tMatrix=(double*)mxGetPr(plhs[3]);
    dims[0]=QSize;
    dims[1]=4;
    plhs[4] = mxCreateDoubleMatrix(QSize,4,mxREAL);
    t2eMatrix=(double*)mxGetPr(plhs[4]);
    plhs[5] = mxCreateDoubleMatrix(QSize,3,mxREAL);
    fpMatrix=(double*)mxGetPr(plhs[5]);
    plhs[6] = mxCreateDoubleMatrix(rowList.size(),1,mxREAL);
    rowVector=(double*)mxGetPr(plhs[6]);
    plhs[7] = mxCreateDoubleMatrix(colList.size(),1,mxREAL);
    colVector=(double*)mxGetPr(plhs[7]);
    plhs[8] = mxCreateDoubleMatrix(valList.size(),1,mxREAL);
    valVector=(double*)mxGetPr(plhs[8]);
    cout<<"HF: "<<HF<<endl;
    for (int i=0;i<EH.rows();i++){
        
        int currH = (EH(i,0)==-1 ? EH(i,1) : EH(i,0));
        
         cout<<"currH: "<<currH<<endl;
 
        epMatrix[i]=EdgePoints(i,0);
        epMatrix[i+EV.rows()]=EdgePoints(i,1);
        epMatrix[i+2*EV.rows()]=EdgePoints(i,2);
        elistMatrix[i]=HV(currH)+1;
        elistMatrix[i+EV.rows()]=HV(nextH(currH))+1;
        cout<<"HV(currH): "<<HV(currH)<<endl;
        cout<<"HV(nextH(currH)): "<<HV(nextH(currH))<<endl;
        e2tMatrix[i]=HF(currH)+1;
        if (twinH(currH)!=-1)
            e2tMatrix[i+EV.rows()]=-(HF(twinH(currH))+1);
        else
            e2tMatrix[i+EV.rows()]=0;
        
        /*if ((Halfedges[i]->face+Halfedges[i]->inface*QSize<0)||(Halfedges[i]->face+Halfedges[i]->inface*QSize>4*QSize-1))
            int kaka=9;*/
        //continue;
        /*t2eMatrix[EF(i,0)+EFi(i,0)*QSize]=i+1;
        if (twinH(currH)!=-1)
            t2eMatrix[EF(twinH(currH))+EFi(twinH(currH))*QSize]=i+1;*/
    }
    for (int i=0;i<QSize;i++)
        for (int j=0;j<4;j++)
            t2eMatrix[i+j*QSize] = FE(i,j)+1;
     cout<<"Done EMatrices: "<<endl;
    for (int i=0;i<QSize;i++){
        fpMatrix[i]=FacePoints(i,0);
        fpMatrix[i+QSize]=FacePoints(i,1);
        fpMatrix[i+2*QSize]=FacePoints(i,2);
    }
    for (int i=0;i<XSize;i++){
        vpMatrix[i]=VertexPoints(i,0);
        vpMatrix[i+XSize]=VertexPoints(i,1);
        vpMatrix[i+2*XSize]=VertexPoints(i,2);
    }
    
    for (int i=0;i<rowList.size();i++){
        rowVector[i]=rowList[i];
        colVector[i]=colList[i];
        valVector[i]=valList[i];
    }
    
    cout<<"Done!"<<endl;
}
