% TEST SCRIPT FOR CC SUBDIVISION TIMINGS

clc;
clearvars;

addpath('ccsubdivisionmatlab');

% params
subdivision_levels = 3;

% read mesh
mesh = MESHQ('bob_quad');

for i=1:subdivision_levels
    
    % subdivide
    disp(['subdividing to level ', num2str(i), '/', num2str(subdivision_levels)]);
    fprintf('input nv = %d\n', mesh.nv);
    tic;
    [VertexPointsTest, EdgePointsTest, ~, ~, ~, FacePointsTest, ~,~,~]=GetCCMatrix(mesh.vertices,double(mesh.quads));
    
    toc;
    
    tic;
    [VertexPoints, EdgePoints, Elist, e2t, t2e, FacePoints, I,J,V]=GetCCMatrixEigen(mesh.vertices,double(mesh.quads));
    toc;
    fprintf('output nv = %d\n\n', mesh.nv+mesh.ne+mesh.nf);
    
    vertexError = max(abs(VertexPointsTest-VertexPoints))
    faceError = max(abs(FacePointsTest-FacePoints))
    
    % build S_0
    S_0 = sparse(I+1,J+1,V, mesh.nv+mesh.ne+mesh.nf, mesh.nv);
    
    % arrange new vertices
    subdivision_vertices =[VertexPoints;EdgePoints;FacePoints];
    
    % arrange new quads
    subdivision_quads = [mesh.quads(:,1) mesh.nv+abs(t2e(:,1)) mesh.nv+mesh.ne+(1:mesh.nf)' mesh.nv+abs(t2e(:,4));...
        mesh.quads(:,2) mesh.nv+abs(t2e(:,2)) mesh.nv+mesh.ne+(1:mesh.nf)' mesh.nv+abs(t2e(:,1));...
        mesh.quads(:,3) mesh.nv+abs(t2e(:,3)) mesh.nv+mesh.ne+(1:mesh.nf)' mesh.nv+abs(t2e(:,2));...
        mesh.quads(:,4) mesh.nv+abs(t2e(:,4)) mesh.nv+mesh.ne+(1:mesh.nf)' mesh.nv+abs(t2e(:,3))];
    
    % return the subdivided mesh
    mesh = MESHQ(subdivision_vertices, subdivision_quads);
    
end
