function wkm = WKM(lbo_vecs, e_vec, sigma, log_eigenvals, landmarks) % compute WKM
% WKM  Computing the Wave Kernel Map for the shape with the
% given landmrks

% compute exponential weights per energy
exponential_coefs_matrix = exp(-(e_vec - log_eigenvals).^2/(2*sigma.^2));

% normalization factors per energy (using broadcast - row vector minus column vector)
Ce = sum(exponential_coefs_matrix).^-1;

% compute the WKM
wkm = cell(1, numel(landmarks));
for i = 1:numel(landmarks)
    % get WKM (using broadcasting e.g. 3x3 matrix time 1x3 row
    % vector)
    wkm{i} = lbo_vecs.*lbo_vecs(landmarks(i), :) *(exponential_coefs_matrix.*Ce);
end

end
