function wkm = WKM_fast(lbo_vecs, e_vec, sigma, log_eigenvals, landmarks, flag_landmarks, S_to_fine) % compute WKM
% WKM  Computing the Wave Kernel Map cofficients for the shape with the
% given landmrks

% compute exponential weights per energy
exponential_coefs_matrix = exp(-(e_vec - log_eigenvals).^2/(2*sigma.^2));

% normalization factors per energy (using broadcast - row vector minus column vector)
Ce = sum(exponential_coefs_matrix).^-1;

% compute the WKM eigens cofficients
wkm = cell(1, numel(landmarks));

if flag_landmarks == 0 % coarse
    for i = 1:numel(landmarks)
        % get WKM eigens cofficients, using broadcasting
        wkm{i} = lbo_vecs(landmarks(i), :)' .* (exponential_coefs_matrix.*Ce);
    end
else % fine linear combination using subdivision matrix to finest level
    for i = 1:numel(landmarks)
        lbo_vec_landmark_fine_linear_comb = S_to_fine(landmarks(i), :)*lbo_vecs;
        % get WKM eigens cofficients, using broadcasting
        wkm{i} = lbo_vec_landmark_fine_linear_comb' .* (exponential_coefs_matrix.*Ce);
    end
end

end
