function C = compute_fmap(F_1, F_2, eigenvals_1, eigenvals_2, alpha1, alpha2, alpha3, alpha4, symmetry_1, symmetry_2, commuting_1, commuting_2)
%% Compute matrices

% get hierarchy eigenvalues matrices
k_1 = numel(eigenvals_1);
k_2 = numel(eigenvals_2);
lambda_1 = spdiags(eigenvals_1, 0, k_1, k_1);
lambda_2 = spdiags(eigenvals_2, 0, k_2, k_2);
symmetry_1 = spdiags(spdiags(symmetry_1, -1:1), -1:1, k_1, k_1); % take only 3 leading diagonals
symmetry_2 = spdiags(spdiags(symmetry_2, -1:1), -1:1, k_2, k_2); % take only 3 leading diagonals

%% Solve for C
% prepare Hessian for each term
% descriptors presveration
h1 = alpha1*kron(F_2*F_2', speye(k_1));
% laplcian commutation
h2 = alpha2*( kron(speye(k_2), lambda_1.^2) -2*kron(lambda_2, lambda_1) + kron(lambda_2.^2, speye(k_1)) );
% symmetry commutation
h3 = alpha3*( kron(speye(k_2), symmetry_1'*symmetry_1) -kron(symmetry_2',symmetry_1') -kron(symmetry_2, symmetry_1) + kron(symmetry_2*symmetry_2', speye(k_1)) );

if alpha4 > 0
    % descriptors commuatation (multiplication)
    disp('Prepare commuting descriptors Hessian');
    X2i_X2i_t_sum = zeros(k_2);
    X1i_t_X1i_sum = zeros(k_1);
    h4 = zeros(k_1*k_2);
    for i = 1:numel(commuting_1) % informative part, fastet than H = H + kron(speye(k_M), Yi'*Yi) - kron(Xi', Yi') - kron(Xi, Yi) + kron(Xi*Xi', speye(k_N));
        X2i = commuting_2{i};
        X1i = commuting_1{i};
        X2i_X2i_t_sum = X2i_X2i_t_sum + X2i*X2i';
        X1i_t_X1i_sum = X1i_t_X1i_sum + X1i'*X1i;
        Xi_kron_Yi = kron(X2i, X1i);
        h4 = h4 - alpha4*(Xi_kron_Yi' + Xi_kron_Yi);
    end
    h4 = h4 + alpha4*(kron(speye(k_2), X1i_t_X1i_sum) + kron(X2i_X2i_t_sum, speye(k_1)));
    H = h1 + h2 + h3 + h4;
else
    H = h1 + h2 + h3;
end

grad = -alpha1*reshape(F_1*F_2', k_1*k_2, 1);
fprintf('Solving linear equation of %d variables \n', numel(grad));
Vec_C = H \ -grad;
fmap_solution_norm = norm(H*Vec_C + grad);
assert(fmap_solution_norm < 1e-10, 'Error: Functional map linear solve norm is above threshold')
C = reshape(Vec_C, k_1, k_2);
% figure; imagesc(C); colormap(bluewhitered);

end
