function C = compute_fmap_cg(F_1, F_2, eigenvals_1, eigenvals_2, alpha1, alpha2, alpha3, alpha4, symmetry_1, symmetry_2, commuting_1, commuting_2, C_0)
%% Compute matrices

% get initial guess
vec_C_0 = C_0(:);

% get hierarchy eigenvalues matrices
k_1 = numel(eigenvals_1);
k_2 = numel(eigenvals_2);
lambda_1 = spdiags(eigenvals_1, 0, k_1, k_1);
lambda_2 = spdiags(eigenvals_2, 0, k_2, k_2);
symmetry_1 = spdiags(spdiags(symmetry_1, -1:1), -1:1, k_1, k_1); % take only 3 leading diagonals
symmetry_2 = spdiags(spdiags(symmetry_2, -1:1), -1:1, k_2, k_2); % take only 3 leading diagonals

% define the anonymous function
A = @(vec_C) reshape( alpha1 * reshape(vec_C, k_1, k_2)*(F_2*F_2') + ...
                      alpha2 * ( (reshape(vec_C, k_1, k_2)*lambda_2   - lambda_1*reshape(vec_C, k_1, k_2))*lambda_2' - lambda_1'*(reshape(vec_C, k_1, k_2)*lambda_2   - lambda_1*reshape(vec_C, k_1, k_2)) ) + ...  
                      alpha3 * ( (reshape(vec_C, k_1, k_2)*symmetry_2 - symmetry_1*reshape(vec_C, k_1, k_2))*symmetry_2' - symmetry_1'*(reshape(vec_C, k_1, k_2)*symmetry_2   - symmetry_1*reshape(vec_C, k_1, k_2)) ) +...    
                      alpha4 * sum(reshape(cell2mat(cellfun(@(O1, O2) (reshape(vec_C, k_1, k_2)*O2 - O1*reshape(vec_C, k_1, k_2))*O2' - O1'*(reshape(vec_C, k_1, k_2)*O2   - O1*reshape(vec_C, k_1, k_2)), commuting_1, commuting_2, 'UniformOutput', false)), k_1, k_2, []), 3), ...
                      k_1*k_2, 1);
% b vector   
b = alpha1 * reshape(F_1*F_2', k_1*k_2, 1);
[Vec_C,fl0,rr0,it0,rv0] = pcg(A,b,1e-8,100, [], [], vec_C_0);
figure;
semilogy(0:it0,rv0/norm(b),'b.');
legend('No Preconditioner');
xlabel('iteration number');
ylabel('relative residual');
hold off;

C = reshape(Vec_C, k_1, k_2);
figure; imagesc(C); colormap(bluewhitered);

end
