function [symmetry_F, symmetry_matrix, symmetry_nn, symmetry_dist] = compute_symmetry(shape, shape_prev, S_0, eigenfunctions, eigenfunctions_pinv, symmetry_plane, prev_symmetry)
% Given a symmetric function and the symmetry axis returns the symmetry

if ~isempty(symmetry_plane) % coarse level
    
    % set symmetry plane for reflection
    switch symmetry_plane
        case 1 % y-z plane
            coordiante_swap_vec = [-1, 1, 1];
        case 2 % x-z plane
            coordiante_swap_vec = [1, -1, 1];
        case 3 % x-y plane
            coordiante_swap_vec = [1, 1, -1];
    end
    % compute by nearest nieghbor of plane reflection
    [symmetry_nn, symmetry_dist] = knnsearch(shape.vertices, shape.vertices .* coordiante_swap_vec, 'K', 1);
    
    % verify bijectivity - we form a permuatation matrix
    if numel(unique(symmetry_nn)) <= shape.nv *0.95 % allow for some deviation
        
        % rotate around prinicpal axis and check all 3 planes
        R = pca(shape.vertices);
        vertices_Rot = shape.vertices*R;
        
        for i = 1:3
            switch i
                case 1 % y-z plane
                    coordiante_swap_vec = [-1, 1, 1];
                case 2 % x-z plane
                    coordiante_swap_vec = [1, -1, 1];
                case 3 % x-y plane
                    coordiante_swap_vec = [1, 1, -1];
            end
            [symmetry_nn, symmetry_dist] = knnsearch(vertices_Rot, vertices_Rot .* coordiante_swap_vec, 'K', 1);
            if numel(unique(symmetry_nn)) == shape.nv
                break
            end
        end
        
        % verify bijectivity - we form a permuatation matrix
        assert(numel(unique(symmetry_nn)) == shape.nv, 'Symmetry Failed');        
    end
    
else
    
    % compute by nearest nieghbor of subdivided symmetry mapped coordinates
    [symmetry_nn, symmetry_dist] = knnsearch(shape.vertices, S_0*prev_symmetry*shape_prev.vertices, 'K', 1);
    
    % verify bijectivity - we form a permuatation matrix
%     assert(numel(unique(symmetry_nn)) == shape.nv, 'Symmetry Failed');
end

% build the operators of self-symmetry
symmetry_matrix = sparse(1:shape.nv, symmetry_nn, 1);

% get functional representation
symmetry_F = eigenfunctions_pinv * symmetry_matrix * eigenfunctions;

end
