function [distortion, distortion_per_face] = map_distortion(shape_1, shape_2, P_12)
% compute mapping distortion by averaging quads distortion
% computing affine 3D transform between each orginial and mapped quad and
% measuring some funciton of the affine trasnformation singular values

% get original vertices
V1 = shape_1.vertices;

% get deformed mapped vertices
V1_deformed = P_12 * shape_2.vertices;

% get quads and quads area
quads = shape_1.quads;
quads_area = shape_1.fa;

% initate Error
E = zeros(shape_1.nf, 1);

% run over quads and measure wieghted distortion
parfor i = 1:shape_1.nf
    
    % get original quads vertices and make homogenues
    v1 = [V1(quads(i,:),:)'; 1 1 1 1];
    
    % get deofrmed quad
    v2 = V1_deformed(quads(i,:),:)';
    
    % solve for the affine transform
    T_vec = kron(v1', eye(3)) \ v2(:);
    
    % get matrix form of the affine transformation
    T_mat = reshape(T_vec, 3, 4);
    
    % compute singular values
    s = svd(T_mat(1:3, 1:3));
    
    % compute weigthed E(s);
    Es = (s(1) - 1)^2 + (s(2) - 1)^2;
    E(i) = Es*quads_area(i);
    
end

% normalize by total area and by total number of vertices
distortion = sum(E)/sum(quads_area)/size(V1,1);
distortion_per_face = E./sum(E);

end
