function transfer_texture(m1, m2, P12, out_dir, mname1, im_2_name)
% Transfer texture between m2 to m1

%% UV1 <- X1 <- X2 <- UV2 
% UV1 <- X1
q1uv = m1.quads_uv(:);
q1 = m1.quads(:);
x2uv_1 = sparse(q1uv, q1, ones(length(q1),1), max(q1uv), max(q1));
% Every vertex may belong to multiple quads
x2uv_1 = spdiags(1./sum(x2uv_1,2),0,m1.nv_uv,m1.nv_uv)*x2uv_1;

% X2 <- UV2
q2uv = m2.quads_uv(:);
q2 = m2.quads(:);
uv2x_2 = sparse(q2, q2uv, ones(length(q2),1), max(q2), max(q2uv));
% Might get pixel value from multiple points, use average
uv2x_2 = spdiags(1./sum(uv2x_2,2),0,m2.nv,m2.nv)*uv2x_2;

% P12 X1 <- X2
P12_uv = x2uv_1 * P12 * uv2x_2;

%% Show quads problem
% m1_d = copy(m1);
% m1_d.uv = P12_uv * m2.uv;
% figure; MESH_VIS.texture(m1_d);

%% Compute bad quads as those that contain vertices mapped to boundary k-ring
uv_boundary_AM = m2.AM_uv - m2.AM_uv'; % deleting duplicated (non boundary) edges
uv_boundary_AM(uv_boundary_AM < 0) = 0; % getting just one direction
grph = digraph(uv_boundary_AM);

% get the grpah degree - 1 for each edge node, 0 otherwise
deg = indegree(grph);

% get boundary vertices
m2_uv_b_verts = find(deg>0);

% get k-ring of uv boundary vertices
k = 1;
m2_uv_k_ring_b = cell(numel(m2_uv_b_verts),1);
uv_AM_sym = m2.AM_uv + m2.AM_uv' + speye(m2.nv_uv);
uv_k_ring = (uv_AM_sym^k);
assert(issymmetric(uv_AM_sym));
for i = 1:numel(m2_uv_b_verts)
    vtx_n = find(uv_k_ring(:,m2_uv_b_verts(i)));
    m2_uv_k_ring_b{i} = vtx_n;
end

% k-ring boundary indicator function
m2_k_ring_verts = unique(cell2mat(m2_uv_k_ring_b));
m2_uv_k_ring_b_func = zeros(m2.nv_uv,1);
m2_uv_k_ring_b_func(m2_k_ring_verts) = 1;

% map the boundary function
m1_uv_mapped_b = P12_uv * m2_uv_k_ring_b_func;


%% decide on bad quads in source - cuts of texture map
[quads_bad, ~] = find(m1_uv_mapped_b(m1.quads_uv));
quads_bad = unique(quads_bad);

% Remove all bad quads
m1_d = copy(m1);
m1_d.uv = P12_uv * m2.uv;

% delete bad quads
m1_d.quads_uv(quads_bad,:) = [];
% figure; MESH_VIS.texture(m1_d);
% title(sprintf('delete k-ring, k = %d', k));


%% Compute target boundary quads for MASK
uv_boundary_AM = m1.AM_uv - m1.AM_uv'; % deleiting duplicated (non boundary) edges
uv_boundary_AM(uv_boundary_AM < 0) = 0; % getting just one direction
grph = digraph(uv_boundary_AM);

% get the grpah degree - 1 for each edge node, 0 otherwise
deg = indegree(grph);

% get boundary quads
[m1_bound_quads, ~] = find(deg(m1.quads_uv));


%% Prepare MASKS for inpainting procedure

% Mask for bad uv mapped quads
m1_mask_uv_bad_quads = copy(m1);
quads_mask = quads_bad;
m1_mask_uv_bad_quads.quads_uv(setdiff(1:(m1.nf), quads_mask),:) = [];
% figure; MESH_VIS.texture(m1_mask_uv_bad_quads); title('Mask bad uv quads');

% Mask for extended padding quads
m1_mask_uv_bound = copy(m1);
quads_mask = unique(m1_bound_quads(:));
m1_mask_uv_bound.quads_uv(setdiff(1:m1.nf, quads_mask),:) = [];
% figure; MESH_VIS.texture(m1_mask_uv_bound); title('Mask extension');

% Write 
MESH_IO.wobj([out_dir, mname1, '_texture_uv.obj'], [m1.uv, zeros(size(m1.uv,1),1)], m1_d.quads_uv, m1_d.uv, m1_d.quads_uv, im_2_name)
MESH_IO.wobj([out_dir, mname1, '_texture_uv_mask.obj'], [m1_mask_uv_bad_quads.uv, zeros(size(m1_mask_uv_bad_quads.uv,1),1)], m1_mask_uv_bad_quads.quads_uv, m1_mask_uv_bad_quads.uv, m1_mask_uv_bad_quads.quads_uv, 'red_im.jpg');
% MESH_IO.wobj([out_dir, mname1, '_texture_uv_map_k', num2str(k), '_mask_ext.obj'], [m1_mask_uv_bound.uv, zeros(size(m1_mask_uv_bound.uv,1),1)], m1_mask_uv_bound.quads_uv, m1_mask_uv_bound.uv, m1_mask_uv_bound.quads_uv, 'r.jpg');

end
